package lcsb.mapviewer.converter.model.celldesigner.geometry;

import static org.junit.Assert.assertEquals;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import lcsb.mapviewer.common.EventStorageLoggerAppender;
import lcsb.mapviewer.common.geometry.LineTransformation;
import lcsb.mapviewer.converter.model.celldesigner.geometry.helper.CellDesignerEllipseTransformation;
import lcsb.mapviewer.converter.model.celldesigner.geometry.helper.CellDesignerPolygonTransformation;
import lcsb.mapviewer.converter.model.celldesigner.geometry.helper.CellDesignerRectangleTransformation;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.model.map.species.Species;

public class AbstractCellDesignerAliasConverterTest {
  Logger logger = Logger.getLogger(AbstractCellDesignerAliasConverterTest.class);

  AbstractCellDesignerAliasConverter<Species> converter;

  @SuppressWarnings("unchecked")
  @Before
  public void setUp() {
    converter = Mockito.spy(AbstractCellDesignerAliasConverter.class);
  }

  @Test
  public void testGetAnchorPointCoordinatesForInvalidImplementation() {
    EventStorageLoggerAppender appender = new EventStorageLoggerAppender();
    Logger.getRootLogger().addAppender(appender);
    try {
      Protein alias = new GenericProtein("id");
      alias.setWidth(1);
      alias.setHeight(1);
      converter.getAnchorPointCoordinates(alias, 0);
      assertEquals(1, appender.getWarnings().size());
    } finally {
      Logger.getRootLogger().removeAppender(appender);
    }
  }

  @Test
  public void testGetters() {
    CellDesignerEllipseTransformation ellipseTransformation = new CellDesignerEllipseTransformation();
    LineTransformation lineTransformation = new LineTransformation();
    CellDesignerPolygonTransformation polygonTransformation = new CellDesignerPolygonTransformation();
    CellDesignerRectangleTransformation rectangleTransformation = new CellDesignerRectangleTransformation();
    converter.setEllipseTransformation(ellipseTransformation);
    converter.setLineTransformation(lineTransformation);
    converter.setPolygonTransformation(polygonTransformation);
    converter.setRectangleTransformation(rectangleTransformation);

    assertEquals(ellipseTransformation, converter.getEllipseTransformation());
    assertEquals(lineTransformation, converter.getLineTransformation());
    assertEquals(polygonTransformation, converter.getPolygonTransformation());
    assertEquals(rectangleTransformation, converter.getRectangleTransformation());
  }

}
