package lcsb.mapviewer.converter.model.celldesigner.geometry;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.awt.geom.Point2D;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.mockito.Mockito;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.converter.model.celldesigner.geometry.helper.CellDesignerAnchor;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Protein;

public class ProteinCellDesignerAliasConverterTest {
  Logger logger = Logger.getLogger(ProteinCellDesignerAliasConverterTest.class);

  ProteinCellDesignerAliasConverter converter = new ProteinCellDesignerAliasConverter(false);

  @Test
  public void testNotImplementedMethod() {
    try {
      Protein alias = Mockito.mock(Protein.class);
      converter.getBoundPathIterator(alias);
      fail("Exception expected");
    } catch (NotImplementedException e) {
      assertTrue(e.getMessage().contains("Not implemented protein converter for type"));
    }
  }

  @Test
  public void testGetPointCoordinates() {
    try {
      Protein alias = Mockito.spy(Protein.class);
      alias.setWidth(10);
      alias.setHeight(10);
      converter.getPointCoordinates(alias, CellDesignerAnchor.E);
      fail("Exception expected");
    } catch (NotImplementedException e) {
      assertTrue(e.getMessage().contains("Unknown type"));
    }
  }

  @Test
  public void testGetAngleForPoint() {
    Protein protein = new GenericProtein("id");
    protein.setWidth(200);
    protein.setHeight(30);
    double angle = 0;
    while (angle <= Math.PI * 2) {
      Point2D point = converter.getResidueCoordinates(protein, angle);
      double newAngle = converter.getAngleForPoint(protein, point);
      assertEquals(angle, newAngle, Configuration.EPSILON);

      angle += 0.1;
    }
  }


}
