package lcsb.mapviewer.converter.model.sbgnml;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.awt.geom.Point2D;

import org.apache.log4j.Logger;
import org.junit.Test;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.IConverter;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Complex;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.field.Residue;

public class SbgnmlXmlParserTest2 {

  Logger logger = Logger.getLogger(SbgnmlXmlParserTest2.class.getName());

  @Test
  public void testAdjustModificationCoordinates() throws Exception {
    SbgnmlXmlParser parser = new SbgnmlXmlParser();
    GenericProtein protein = new GenericProtein("id");
    protein.setWidth(50);
    protein.setHeight(30);
    protein.setX(200);
    protein.setX(300);
    Residue mr = new Residue();
    Point2D position = new Point2D.Double(100, 20);
    mr.setPosition(position);
    protein.addResidue(mr);
    parser.adjustModificationCoordinates(protein);

    assertTrue(mr.getPosition().distance(position) > Configuration.EPSILON);

  }

  @Test
  public void createModelWithCompartmentsTest() throws Exception {
    try {
        IConverter converter = new SbgnmlXmlConverter();

        Model model = converter
            .createModel(new ConverterParams().filename("testFiles/sbgnmlParserTestFiles/sbgnmlFiles/elements_inside_compartment.xml"));
        Complex complexInsideCompartment = model.getElementByElementId("csa1830");
        assertNotNull("Complex inside compartment has undefined compartment", complexInsideCompartment.getCompartment());
        Complex complexOutsideCompartment = model.getElementByElementId("csa1831");
        assertNull("Complex outside compartment has not null compartment",complexOutsideCompartment.getCompartment());
        Compartment compartment = model.getElementByElementId("ca107");
        assertNull("Top compartment has not null compartment", compartment.getCompartment());
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

}
