package lcsb.mapviewer.converter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.converter.zip.ImageZipEntryFile;
import lcsb.mapviewer.converter.zip.LayoutZipEntryFile;
import lcsb.mapviewer.converter.zip.ZipEntryFile;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.map.model.Model;

public class ProjectFactory {

  /**
   * Deafult class clogger.
   */
  @SuppressWarnings("unused")
  private Logger logger = Logger.getLogger(ProjectFactory.class);

  private ComplexZipConverter converter;

  public ProjectFactory(ComplexZipConverter converter) {
    this.converter = converter;
  }

  public Project create(ComplexZipConverterParams params) throws InvalidInputDataExecption {
    return create(params, new Project());
  }

  public Project create(ComplexZipConverterParams params, Project project) throws InvalidInputDataExecption {
    try {
      Model model = converter.createModel(params);

      Set<Model> models = new HashSet<>();
      models.add(model);
      models.addAll(model.getSubmodels());

      project.addModel(model);

      ZipFile zipFile = params.getZipFile();
      Enumeration<? extends ZipEntry> entries;

      entries = zipFile.entries();
      List<ImageZipEntryFile> imageEntries = new ArrayList<>();
      List<Layout> layouts = new ArrayList<>();
      int overlayOrder = 1;
      while (entries.hasMoreElements()) {
        ZipEntry entry = entries.nextElement();
        if (!entry.isDirectory()) {
          ZipEntryFile zef = params.getEntry(entry.getName());
          if (zef instanceof ImageZipEntryFile) {
            imageEntries.add((ImageZipEntryFile) zef);
          } else if (zef instanceof LayoutZipEntryFile) {
            project.addLayout(
                converter.layoutZipEntryFileToLayout(params, zipFile, entry, (LayoutZipEntryFile) zef, overlayOrder++));
          }
        }
      }
      if (imageEntries.size() > 0) {
        OverviewParser parser = new OverviewParser();
        project
            .addOverviewImages(parser.parseOverviewLinks(models, imageEntries, params.getVisualizationDir(), zipFile));
      }
      return project;
    } catch (IOException e) {
      throw new InvalidArgumentException(e);
    } catch (InvalidCoordinatesFile e) {
      throw new InvalidInputDataExecption("Invalid coordinates file. " + e.getMessage(), e);
    }
  }
}
