package lcsb.mapviewer.model.map.reaction.type;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.apache.commons.lang3.SerializationUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;

public class UnknownTransitionReactionTest {

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testSerialization() {
    try {
      SerializationUtils.serialize(new UnknownTransitionReaction());
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testConstructorWithInvalidArg() {
    try {
      Reaction reaction = new Reaction();
      new UnknownTransitionReaction(reaction);
      fail("Exception expected");
    } catch (InvalidArgumentException e) {

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testConstructorWithInvalidArg2() {
    try {
      Reaction reaction = new Reaction();
      reaction.addProduct(new Product());
      new UnknownTransitionReaction(reaction);
      fail("Exception expected");
    } catch (InvalidArgumentException e) {

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testConstructor() {
    try {
      Reaction reaction = new Reaction();
      reaction.addProduct(new Product());
      reaction.addReactant(new Reactant());
      UnknownTransitionReaction validReaction = new UnknownTransitionReaction(reaction);
      assertNotNull(validReaction);

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testCopy() {
    try {
      UnknownTransitionReaction original = new UnknownTransitionReaction();
      original.addProduct(new Product());
      original.addReactant(new Reactant());
      original.addReactant(new Reactant());
      UnknownTransitionReaction product = original.copy();
      assertNotNull(product);
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testInvalidCopy() {
    try {
      Mockito.spy(new UnknownTransitionReaction()).copy();
      fail("Exception expected");
    } catch (NotImplementedException e) {
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testGetters() {
    try {
      UnknownTransitionReaction original = new UnknownTransitionReaction();
      assertNotNull(original.getReactionRect());
      assertNotNull(original.getStringType());
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
}
