package lcsb.mapviewer.api;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.CALLS_REAL_METHODS;

import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.annotation.services.MiriamConnector;
import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;

public class BaseRestImplTest extends RestTestFunctions {
  Logger logger = Logger.getLogger(BaseRestImplTest.class);

  @Autowired
  MiriamConnector mc;

  @Autowired
  PubmedParser pubmedParser;

  @Test
  public void testMathMLToPresentationML() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    String content = super.readFile("testFiles/mathml/content.xml");
    String presentationXml = controller.mathMLToPresentationML(content);
    assertNotNull(presentationXml);
    assertTrue("Presentation math ml doesn't contain expected mrow tag", presentationXml.indexOf("mrow") >= 0);
  }

  @Test
  public void testMathMLToPresentationMLCalledTwice() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    String content = super.readFile("testFiles/mathml/content.xml");
    String presentationXml = controller.mathMLToPresentationML(content);
    String presentationXml2 = controller.mathMLToPresentationML(content);
    assertEquals(presentationXml, presentationXml2);
  }

  @Test
  public void testMathMLToPresentationMLCalledTwiceShouldntHaveXmlNode() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    String content = super.readFile("testFiles/mathml/content.xml");
    String presentationXml = controller.mathMLToPresentationML(content);
    assertFalse(presentationXml.startsWith("<?xml"));
  }

  @Test
  public void testCreateAnnotationWithWhitespace() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    controller.setMiriamConnector(mc);
    controller.setPubmedParser(pubmedParser);
    Map<String, Object> response = controller.createAnnotation(new MiriamData(MiriamType.PUBMED, "28255955 "));
    assertNotNull(response);
  }

  @Test
  public void testCreateAnnotationWithNonNumericValues() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    controller.setMiriamConnector(mc);
    controller.setPubmedParser(pubmedParser);
    Map<String, Object> response = controller.createAnnotation(new MiriamData(MiriamType.PUBMED, "28255955PG"));
    assertNotNull(response);
  }

  @Test
  public void testCreateAnnotationWithEmptyValue() throws Exception {
    BaseRestImpl controller = Mockito.mock(BaseRestImpl.class, CALLS_REAL_METHODS);
    controller.setMiriamConnector(mc);
    controller.setPubmedParser(pubmedParser);
    Map<String, Object> response = controller.createAnnotation(new MiriamData(MiriamType.PUBMED, ""));
    assertNotNull(response);
  }
}
