package lcsb.mapviewer.web.config;

import java.io.InputStream;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Properties;

import javax.servlet.FilterRegistration;
import javax.servlet.ServletContext;
import javax.servlet.ServletRegistration;
import javax.servlet.SessionCookieConfig;
import javax.servlet.SessionTrackingMode;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.primefaces.webapp.filter.FileUploadFilter;
import org.springframework.security.web.session.HttpSessionEventPublisher;
import org.springframework.web.WebApplicationInitializer;
import org.springframework.web.context.ContextLoaderListener;
import org.springframework.web.context.request.RequestContextListener;
import org.springframework.web.context.support.AnnotationConfigWebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;

import lcsb.mapviewer.annotation.SpringAnnotationConfig;
import lcsb.mapviewer.api.SpringRestApiConfig;
import lcsb.mapviewer.persist.SpringPersistConfig;
import lcsb.mapviewer.services.SpringServiceConfig;
import lcsb.mapviewer.web.bean.utils.ApiAccessControlFilter;
import lcsb.mapviewer.web.bean.utils.JsfAjaxAccessControlAllowFilter;
import lcsb.mapviewer.web.bean.utils.XFrameAccessControlFilter;

public class WebAppInitializer implements WebApplicationInitializer {
  Logger logger = Logger.getLogger(WebAppInitializer.class);

  @Override
  public void onStartup(ServletContext container) {

    AnnotationConfigWebApplicationContext rootContext = new AnnotationConfigWebApplicationContext();
    rootContext.register(
        SpringAnnotationConfig.class,
        SpringPersistConfig.class,
        SpringRestApiConfig.class,
        SpringServiceConfig.class);

    container.addListener(new ContextLoaderListener(rootContext));
    container.addListener(SessionListener.class);
    container.addListener(HttpSessionEventPublisher.class);
    container.addListener(RequestContextListener.class);

    container.setSessionTrackingModes(new HashSet<>(Arrays.asList(SessionTrackingMode.COOKIE)));

    assert container.setInitParameter("com.sun.faces.enableMissingResourceLibraryDetection", "true");
    assert container.setInitParameter("primefaces.THEME", "none");
    assert container.setInitParameter("javax.faces.FACELETS_BUFFER_SIZE", "65535");
    assert container.setInitParameter("facelets.BUFFER_SIZE", "65535");
    assert container.setInitParameter("javax.faces.FACELETS_LIBRARIES", "/WEB-INF/springsecurity.taglib.xml");
    assert container.setInitParameter("javax.faces.CONFIG_FILES", "/WEB-INF/faces-config.xml");
    assert container.setInitParameter("javax.faces.DEFAULT_SUFFIX", ".xhtml");
    assert container.setInitParameter("javax.faces.FACELETS_SKIP_COMMENTS", "true");

    String file = "/WEB-INF/resources/log4j.properties";
    try {
      Properties props = new Properties();
      InputStream configStream = container.getResourceAsStream(file);
      props.load(configStream);
      configStream.close();
      PropertyConfigurator.configure(props);
    } catch (Exception e) {
      logger.error("Problem with loading log4j configuration: " + file);
    }

    AnnotationConfigWebApplicationContext dispatcherContext = new AnnotationConfigWebApplicationContext();
    dispatcherContext.setParent(rootContext);
    dispatcherContext.register(SpringWebConfig.class);

    ServletRegistration.Dynamic dispatcher =
        container.addServlet("dispatcher", new DispatcherServlet(dispatcherContext));
    dispatcher.setLoadOnStartup(1);
    dispatcher.addMapping("/api/*");

    /*
     * ======= FILTERS =======
     */

    FileUploadFilter fileUploadFilter = new FileUploadFilter();
    FilterRegistration.Dynamic fileUploadFilterReg = container.addFilter("PrimeFaces FileUpload Filter",
        fileUploadFilter);
    fileUploadFilterReg.setInitParameter("thresholdSize", "51200");
    fileUploadFilterReg.addMappingForServletNames(null, true, "Faces Servlet");

    XFrameAccessControlFilter xFrameFilter = new XFrameAccessControlFilter();
    FilterRegistration.Dynamic xFrameFilterReg = container.addFilter("xFrameFilter", xFrameFilter);
    xFrameFilterReg.addMappingForUrlPatterns(null, true, "/*");

    JsfAjaxAccessControlAllowFilter jsfAjaxAccessControlAllowFilter = new JsfAjaxAccessControlAllowFilter();
    FilterRegistration.Dynamic jsfAjaxAccessControlAllowFilterReg = container.addFilter("allowAjaxCrossSiteQueries",
        jsfAjaxAccessControlAllowFilter);
    jsfAjaxAccessControlAllowFilterReg.addMappingForUrlPatterns(null, true, "/*");

    ApiAccessControlFilter apiFilter = new ApiAccessControlFilter();
    FilterRegistration.Dynamic apiFilterReg = container.addFilter("apiFilter", apiFilter);
    apiFilterReg.addMappingForUrlPatterns(null, true, "/api/*");


    /*
     * =============== COOKIE SETTINGS ===============
     */
    SessionCookieConfig cookieConfig = container.getSessionCookieConfig();
    cookieConfig.setName("MINERVA_AUTH_TOKEN");
    cookieConfig.setHttpOnly(false);
  }

}
